<?php
/*
 * $Id: paypalnode-admin.inc,v 1.1.2.8 2008/01/04 14:53:14 budda Exp $
 * @name PayPalNode Admin
 * @description Backend administration processes
 * @author Mike Carter <ixis.co.uk/contact>
 *
 */

/*
	paypalnode_settings_form
*/
function paypalnode_settings() {
	$form = array();
	$types = array();
	$vocabs = array();

  $form['paypal'] = array('#type' => 'fieldset', '#title' => t('PayPal Account'), '#description' => t('You should ensure your PayPal account is set up as a business to accept payments in your selected currency.'));

	$form['paypal']['paypalnode_email_account'] = array(
		'#type' => 'textfield',
		'#title' => 'PayPal Account',
		'#description' => 'The email address associated with your PayPal account that will accept the payments.',
		'#default_value' => variable_get('paypalnode_email_account', PAYPALNODE_EMAIL_ACCOUNT),
		'#required' => TRUE
	);

  $form['paypal']['paypalnode_currency'] = array(
   '#type'           => 'select',
   '#title'          => t('Currency'),
   '#default_value'  => variable_get('paypalnode_currency', PAYPALNODE_CURRENCY),
   '#options'        => array('USD'=>'USD', 'GBP'=>'GBP', 'EUR'=>'EUR'),
   '#description' => t('Define the currency the merchant PayPal account accepts.'),
   );

	$form['paypal']['paypalnode_thanks_path'] = array(
		'#type' => 'textfield',
		'#title' => t('Thankyou Page'),
		'#description' => 'The path to a page displaying a thankyou message once the payment has been made. Because the new node will not be published instantly, it is not possible to redirect them to the new content straight away.',
		'#default_value' => variable_get('paypalnode_thanks_path', PAYPALNODE_THANKS_PATH),
		'#required' => TRUE
	);

  $form['paypal']['paypalnode_interstitial'] = array(
   '#type' => 'textarea',
	 '#title' => t('Interstitial Message'),
   '#description' => t('An optional message to display after content submission but before being redirected to PayPal. This could be used to inform the user about what is happening. You can include raw HTML here.'),
   '#rows' => 5,
   '#default_value' => variable_get('paypalnode_interstitial', '')
  );

  $form['content'] = array(
    '#type' => 'fieldset',
    '#title' => t('Site Content'),
    '#description' => t('A single !content can be made charagable when used in combination with vocabulary terms configured in the !fee.', array('!content' => l(t('content-type'), 'admin/content/types'), '!fee' => l(t('content Fees'), 'admin/content/fees')))
    );

	$available_types = node_get_types();
	foreach($available_types as $type) {
		$types[$type->type] = $type->name;
	}
	asort($types);

	$form['content']['paypalnode_content_type'] = array(
    '#type' => 'select',
    '#title' => t('Content Type'),
    '#description' => t('The content type you want to charge users for publishing.'),
    '#options' => $types,
    '#size' => 1,
    '#default_value' => variable_get('paypalnode_content_type', PAYPALNODE_CHARGE_TYPE)
  );

  // List available vocabularies that AREN'T freetagging enabled
	$available_vocabs = taxonomy_get_vocabularies();
	foreach($available_vocabs as $vocab) {
    if(!$vocab->tags) {
  		$vocabs[$vocab->vid] = $vocab->name;
    }
	}

	$form['content']['paypalnode_vocabularies'] = array(
    '#type' => 'select',
    '#title' => t('Vocabularies'),
    '#description' => t('The vocabularies which contain terms that have an associated price. NOTE: Free tagging vocabularies cannot be used. You can manage costs for combinations of terms in the !fee section.', array('!fee' => l(t('Content Fees'), 'admin/content/fees'))),
    '#options' => $vocabs,
    '#size' => 10,
    '#multiple' => TRUE,
    '#default_value' => variable_get('paypalnode_vocabularies', PAYPALNODE_CHARGE_VOCABULARIES)
  );

  $form['expiry'] = array(
    '#type' => 'fieldset',
    '#title' => t('Content Expiration'),
    '#description' => t('Content can be automatically un-published after a certain lifespan specified in days.')
    );

	$form['expiry']['paypalnode_expiry_days'] = array(
		'#type' => 'textfield',
		'#title' => t('Content expiration time'),
    '#field_suffix' => t('day(s)'),
 		'#description' => t('Time specified as a number of days before the content should be unpublished. %never days means the content will never expire.', array('%never' => t('0'))),
		'#default_value' => variable_get('paypalnode_expiry_days', '0'),
    '#size' => 3,
	);

	$form['expiry']['paypalnode_expiry_notify'] = array(
		'#type' => 'checkbox',
		'#title' => t('Notify author'),
 		'#description' => t('Notify author by email about their content expiration.'),
		'#default_value' => variable_get('paypalnode_expiry_notify', '0'),
	);

  $form['expiry']['paypalnode_expiry_email_subject'] = array(
    '#type' => 'textfield',
		'#title' => t('Notification email subject'),
    '#description' => t(''),
		'#default_value' => variable_get('paypalnode_expiry_email_subject', paypalnode_help('notifyemail/subject')),
  );

  $form['expiry']['paypalnode_expiry_email_body'] = array(
    '#type' => 'textarea',
    '#title' => t('Notification email body'),
    '#rows' => 11,
    '#description' => t('If the notify author option is activated (above) then an email will be sent out to the author when the content is unpublished. Available variables are: !username, !site, !site-url, !site-name, !expirydays, !title, !created.'),
    '#default_value' => variable_get('paypalnode_expiry_email_body', paypalnode_help('notifyemail/body'))
  );

	return system_settings_form($form);
}


/*
 * Token module - used to provide token replacements when writing emails.
 */
function paypalnode_token_values($type, $object = NULL) {
  if ($type == 'PayPal Node') {
    $tokens['expirydays'] = variable_get('paypalnode_expiry_days', 'never');
    $tokens['created']    = format_date($object['created'], 'large');
    $tokens['title']      = $object['title'];
    $tokens['username']   = $object['name'];
    return $tokens;
  }
}


/*
 * Token module - used to provide descriptive help about each available token
 * when writing emails.
 */
function paypalnode_token_list($type = 'all') {
  if ($type == 'PayPal Node') {
    $tokens['PayPal Node']['expirydays'] = t("The expiry date for paid content.");
    $tokens['PayPal Node']['created']    = t("Full date of when the content was created.");
    $tokens['PayPal Node']['title']      = t("Title of the content.");
    $tokens['PayPal Node']['username']   = t('Author name of the content.');
    return $tokens;
  }
}


/*
 * Price Admin System
 */
function paypalnode_fee_settings_form() {
  $form = paypalnode_fee_edit_form();
  return $form;
}


function paypalnode_fee_edit_form($terms = '') {
  $op = t('Add cost');

  // Check if there are any term names passed as an argument for editing
  if(arg(4)) {
    $edit_form = paypalnode_fee_select(arg(4));
    $op = t('Update item');
  }

  $form = array();

  $vocabs = variable_get('paypalnode_vocabularies', PAYPALNODE_CHARGE_VOCABULARIES);
  $form['vocab'] = array('#type' => 'fieldset',
                         '#collapsible' => FALSE);

  // List the form elements in reverse order to match Drupals taxonomy form display
  rsort($vocabs);

  foreach($vocabs as $vid) {
    $vocab = taxonomy_get_vocabulary($vid);
    $form['vocab'][$vid] = array(
      '#type' => 'textfield',
      '#title' => $vocab->name,
      '#autocomplete_path' => 'taxonomy/autocomplete/'. $vocab->vid,
      '#maxsize' => 64,
      '#size' => 35,
      '#default_value' => is_array($edit_form['terms'][$vocab->vid]) ? current($edit_form['terms'][$vocab->vid]) : '',
      '#weight' => $vocab->weight,
    );
  }

  $form['price'] = array(
    '#type' => 'textfield',
    '#size' => 10,
    '#maxlength' => 10,
    '#default_value' => number_format($edit_form['price'], 2)
  );

  // Only show  the filter box on the main list page
  if(!$edit_form) {
    $form['filter'] = array(
          '#type' => 'textfield',
          '#size' => 64,
          '#maxlength' => 250,
          '#title' => 'Filter terms',
    );
  }

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => $op,
    '#weight' => 5
  );

  $form['#tree'] = TRUE;

  return $form;
}


function paypalnode_fee_edit_form_submit($form_id, $form_values) {
  paypalnode_fee_settings_form_submit($form_id, $form_values);

  return 'admin/content/fees';
}


function paypalnode_fee_edit_form_validate($form_id, $form_values) {
  // check the terms exist
  $valid = 0;
  $total = 0;

  /* For each vocabulary we ensure the term typed in actually exists
     within that vocabulary */
  foreach($form_values['vocab'] as $vid => $termname) {
    if($termname) {
      $terms = taxonomy_get_term_by_name($termname);
      foreach($terms as $term) {
        if($term->vid == $vid) {
          $valid++;
        }
      }
      $total++;
    }
  }

  if($total != $valid) {
    form_set_error('vocab', t("%valid specified doesn't exist in the corresponding taxonomy vocabulary.", array('%valid' => format_plural($valid, t('1 term'), t('@count terms')) )));
  }
}


/*
 * Clean up the submitted cost entry and save to the database
 */
function paypalnode_fee_settings_form_submit($form_id, $form_values) {
  $terms = array();

  foreach ($form_values['vocab'] as $term) {
    if ($term) {
      $terms[] = '"'.$term.'"';
    }
  }

  paypalnode_fee_insert($terms, $form_values['price']);
}


/*
	Add or Update a cost entry in the database
*/
function paypalnode_fee_insert($terms, $price) {

  $tids = paypalnode_terms_to_tids($terms);

  if ($obj = paypalnode_get_price_object($tids, true)) {
          db_query("UPDATE {paypalnode_prices} SET price = '%f' WHERE pid = '%d'", $price, $obj->pid);
  } else {
          paypalnode_create_price($tids, $price);
  }


}

function paypalnode_create_price($tids, $price) {
  $new_pid = db_next_id('{paypalnode_prices}_pid');

  db_query("INSERT INTO {paypalnode_prices} VALUES('%d', '%f')", $new_pid, $price);

  foreach ($tids as $t) {
          db_query("INSERT INTO {term_prices} (pid, tid) VALUES('%d', '%d')", $new_pid, $t);
  }

  return $new_pid;
}


/*
	paypalnode_fee_select
*/
function paypalnode_fee_select($tids) {
  $entry = array();

  $term_args = explode(',' , $tids);
  if (count($term_args) > 0) {

    // Convert term names in to term ids
    $terms = array();
    foreach ($term_args as $term_name) {
      $term = taxonomy_get_term($term_name);
      $terms[] = $term->tid;
      $entry['terms'][$term->vid][$term->tid] = $term->name;
    }

  }

  // Retrieve matching cost & merge with existing data
  $obj = paypalnode_get_price_object($terms);
  if ($obj) {
          $entry['price'] = $obj->price;
  }


  return $entry;
}


/*
	Delete a cost from the database
*/
function paypalnode_fee_delete($terms) {

  $obj = paypalnode_get_price_object($terms);

  if ($obj) {
        db_query("DELETE FROM {term_prices} WHERE pid='%d'", $obj->pid);
        db_query("DELETE FROM {paypalnode_prices} WHERE pid='%d'", $obj->pid);
        return true;
  }

  return false;

}


function paypalnode_fee_delete_form() {
  $tids = explode(',', arg(4));

  // If no terms provided then redirect
  if(count($tids) < 1) {
    drupal_set_message(t('No entry was specified to delete.'), 'error');
    drupal_goto('admin/content/fees');
  }

  // Convert TermIDs in to a text string
  foreach($tids as $term_id) {
    $term = taxonomy_get_term($term_id);
    $terms[] = $term->name;
  }
  $terms = implode(', ', $terms);

  $form['tids'] = array('#type' => 'value', '#value' => arg(4));
  return confirm_form($form,
    t('Deletion of %title', array('%title' => $terms)),
    'admin/content/fees',
    t('Are you sure you want to delete this cost entry? This action cannot be undone.'),
    t('Delete'), t('Cancel'));
}


function paypalnode_fee_delete_form_submit($form_id, $form_values) {
  if($form_values['op'] == t('Delete') && $form_values['confirm'] == 1) {
		$terms = explode(',', $form_values['tids']);

    // Execute the deletion and check for success
		if(paypalnode_fee_delete($terms)) {
			drupal_set_message('Entry deleted');
		} else {
			drupal_set_message('Deletion failed.', 'error');
		}
	}

  return 'admin/content/fees';
}


/*
	Produce a list of all the current taxonomy combinations and prices
	Used for the theme table display
*/
function paypalnode_fee_list() {
  $i = 0;
  $costs = array();

  $prices = db_query("SELECT * FROM {paypalnode_prices} WHERE price != 0 ORDER BY pid");
  while($row = db_fetch_array($prices)) {

    $tids = array();

    $q = db_query("SELECT tp.tid FROM {term_prices} tp INNER JOIN {term_data} td ON tp.tid = td.tid WHERE tp.pid = '%d' ORDER BY td.vid ASC", $row['pid']);
    while ($r = db_fetch_object($q)) {
        $tids[] = $r->tid;
    }

    foreach($tids as $term_id) {
      $term = taxonomy_get_term($term_id);
      // break this up so that each term is in its own column (sort by vid to match column headers from the theme function below?)
      $costs[$i]['terms'][$term_id] = $term->name;
    }

    $costs[$i]['price'] = number_format($row['price'], 2);

    $i++;
  }

  return $costs;
}


function theme_paypalnode_fee_settings_form($form) {

// change this to use vocabulary names for headers, sort by vid to match above function (see comment)

  $vocabs = variable_get('paypalnode_vocabularies', array(1,2));
  sort($vocabs);
  foreach ($vocabs as $i => $v) {
    $vocab_headers[$v] = taxonomy_get_vocabulary($v)->name;
  }

  $header = array_merge($vocab_headers, array(t('Cost'), array('data' => t('Operations'), 'colspan' => 2)));
  $rows[] = array(array('data' => drupal_render($form['vocab']), 'colspan' => count($vocab_headers)), drupal_render($form['price']), array('data' => drupal_render($form['submit']), colspan => 2));

  $rows[] = array(array('data' => drupal_render($form['filter']), 'colspan' => 4));

  $rowid = 0;

  foreach (paypalnode_fee_list() as $rid => $price) {
    $terms = array();
    $tids = array();

    // Builds a list of comma seperated term ids which match with whats currenctly in the db table

    foreach ($price['terms'] as $tid => $term) {
      $terms[] = array('data' => $term, 'class' => 'paypalnode-fee-terms');
      $tids[] = $tid;
    }

    $termids = implode(',', $tids);

    $rows[] = array('data' => array_merge($terms, array(/*array('data' => implode(',', $price['terms']), 'class' => 'paypalnode-fee-terms'), */$price['price'],
			l(t('edit'), 'admin/content/fees/edit/'. $termids),
			l(t('delete'), 'admin/content/fees/delete/'. $termids),
			)), 'id' => "row{$rowid}");

    $rowid++;
  }

  $path = drupal_get_path('module', 'paypalnode');
  drupal_add_js($path.'/fee-filter.js','module','header');

  $output = drupal_render($form);
  $output .= theme('table', $header, $rows, array('width' => '100%'));

  return $output;
}


function theme_paypalnode_fee_edit_form($form) {
  $header = array(t('Categories'), t('Cost'), array('data' => t('Operations'), 'colspan' => 2));
  $rows[] = array(drupal_render($form['vocab']), drupal_render($form['price']), array('data' => drupal_render($form['submit']), colspan => 2));
  $output = drupal_render($form);
  $output .= theme('table', $header, $rows, array('width' => '100%'));

  return $output;
}


function paypalnode_report_form() {

  $start = arg(3);  // start date might be passed in on the GET string
  $end = arg(4);  // ditto for end date

  $filter_term_tids = explode('-', arg(5));



  $taxonomy_filters = array(
    '#type' => 'fieldset',
    '#title' => t('Category Filter'),
    '#tree' => true,
    '#prefix' => '<div class="container-inline">',
    '#suffix' => '</div>',
  );
  $vocabs = variable_get('paypalnode_vocabularies', array(1,2));

  foreach ($vocabs as $v) {
    $default = 0;
    $voc = taxonomy_get_vocabulary($v);

    $terms = taxonomy_get_tree($v);
    $options = array('0' => '<any>');

    foreach ($terms as $t) {
      $options[$t->tid] = $t->name;
      if (in_array($t->tid, $filter_term_tids)) {
        $default = $t->tid;
      }
    }

    $taxonomy_filters[$v] = array(
      '#type' => 'select',
      '#title' => $voc->name,
      '#options' => $options,
      '#default_value' => $default,
    );
  }

  $now = $end ? getdate($end) : getdate();

  $default_end = array('year' => $now['year'], 'month' => $now['mon'], 'day' => $now['mday']);

  if ($start) {
    $now = getdate($start);
  } else {
    $now['mon'] -= 1;
    if ($now['mon'] == 0) {
      $now['mon'] == 12;
      $now['year'] -= 1;
    }
  }

  $default_start = array('year' => $now['year'], 'month' => $now['mon'], 'day' => $now['mday']);

  $form = array(
    'date_filter' => array(
      '#type' => 'fieldset',
      '#title' => t('Date Filter'),
      '#tree' => false,
      'start_date' => array(
        '#type' => 'date',
        '#title' => t('Start Date'),
        '#default_value' => $default_start,
        '#prefix' => '<div class="container-inline">',
        '#process' => array('_paypalnode_expand_date' => array()),
      ),
      'end_date' => array(
        '#type' => 'date',
        '#title' => t('End Date'),
        '#default_value' => $default_end,
        '#suffix' => '</div>',
        '#process' => array('_paypalnode_expand_date' => array()),
      ),
    ),
    'taxonomy_filters' => $taxonomy_filters,
    'submit' => array(
      '#type' => 'submit',
      '#value' => 'Filter',
      '#weight' => 10,
    ),
  );

  return $form;
}


function paypalnode_report_form_validate($form_id, $form_values) {


  $start = $form_values['start_date'];
  $end = $form_values['end_date'];

  $start_date = mktime(0, 0, 0, $start['month'], $start['day'], $start['year']);
  $end_date = mktime(0, 0, 0, $end['month'], $end['day'], $end['year']);

  if ($end_date <= $start_date) {
    form_set_error('end_date', t('End date must be later than start date'));
  }

}

function paypalnode_report_form_submit($form_id, $form_values) {


  $start = $form_values['start_date'];
  $end = $form_values['end_date'];

  $start_date = mktime(23,59,59, $start['month'], $start['day'], $start['year']);
  $end_date = mktime(23,59,59, $end['month'], $end['day'], $end['year']);

  $taxonomy_string = implode('-', $form_values['taxonomy_filters']);

  return sprintf('admin/content/report/%d/%d/%s', $start_date, $end_date, $taxonomy_string);

}


function theme_paypalnode_report_form($form) {

  $output = '';

  $stats = module_exists('statistics');

  $vocabs = variable_get('paypalnode_vocabularies', array(1,2));
  sort($vocabs);
  $type = variable_get('paypalnode_content_type', PAYPALNODE_CHARGE_TYPE);

  $where_string = ' WHERE n.type = \'%s\' AND n.status = 1';  // by default, we at least want to filter by node type
  $start = arg(3);  // start date might be passed in on the GET string
  $end = arg(4);  // ditto for end date

  $filter_term_tids = explode('-', arg(5));


  $filter_terms = array();

  $filter_by_terms = false;

  foreach ($filter_term_tids as $f) {
    if ($f > 0) {
      $filter_by_terms = true;
      $temp = taxonomy_get_term($f);
      $filter_terms[$temp->vid] = $temp;
    }
  }

  if ($start && $end) {  // if we have both start and end dates, add to WHERE clause
    $where_string .= sprintf(' AND n.created > %d AND n.created < %d', $start, $end);
  }


  foreach ($vocabs as $i => $v) {  // these will be used for column headers
    $vocab_headers[$v] = taxonomy_get_vocabulary($v)->name;
  }


  $count = count($vocabs);

  /**
   * Begin building the query
   */

  $query_select = 'SELECT pn.nid, n.title, u.name, pn.price';
  $query_from = ' FROM {node} n INNER join {users} u ON u.uid = n.uid INNER JOIN {paypalnode_node} pn
    ON pn.nid = n.nid';
  $query_where = $where_string;


  if ($stats) {
    $query_from .= ' LEFT JOIN {node_counter} nc ON nc.nid = n.nid';
    $query_select .= ', nc.totalcount';
  }

  $query = $query_select . $query_from . $query_where;

  $q = db_query($query, $type);

  $rows = array();

  $totals = array();

  while ($r = db_fetch_array($q)) {  // fetch the query results and do some very basic processing
    $filter_out = false;

    $nid = array_shift($r);
    $title = array_shift($r);

    /**
     * Display the associated terms for the node referred to by the current query result
     * There will only be one term for each vocabulary
    */

    $term_columns = array();

    foreach ($vocabs as $v) {
      $term = current(taxonomy_node_get_terms_by_vocabulary($nid, $v));

      if ($filter_by_terms) {
        if (isset($filter_terms[$v])) {
          if ($filter_terms[$v]->tid != $term->tid) {
            $filter_out = true;
          }
        }
      }

      $term_columns[$vocab_headers[$v]] = $term->name;
    }

    $r = array_merge($term_columns, $r);

    if ($filter_out) {
      continue;
    }

    $r = array_merge(array('Title' => $title), $r);

    foreach ($r as $i => $col) {
      if ($col == NULL)  {// display empty cells as '-'
        $r[$i] = '-';
      }

      if (is_numeric($col)) {
        if ($totals[$i] !== '-') {
          $totals[$i] += $col;
        }
      } else {
        $totals[$i] = '-';
      }
    }

    $rows[] = $r;
  }

  $totals['Title'] = 'Total';

  $results = count($rows);

  $rows[] = array('data' => $totals, 'class' => 'paypalnode_report_totals');

  // the array_merge is necessary because some of these components (vocab_headers) are dynamic

  $headers = array_merge(array('Title'), $vocab_headers, array('Creator', 'Price'));

  if ($stats) {
    $headers = array_merge($headers, array('Views'));
  }

  $output .= '<div>';
  $output .= drupal_render($form);
  $output .= '</div>';

  $output .= '<div><strong>' . $results . ' results found</strong></div>';

  $output .= theme('table', $headers, $rows);

  return $output;

}

function _paypalnode_expand_date($element) {

  $first = db_result(db_query("SELECT created FROM {node} WHERE status=1 ORDER BY created ASC LIMIT 1"));
  $first_date = getdate($first);
  $first_year = $first_date['year'];
  $current_date = getdate();
  $current_year = $current_date['year'];

  // Default to current date
  if (empty($element['#value'])) {
    $element['#value'] = array('day' => format_date(time(), 'custom', 'j'),
                            'month' => format_date(time(), 'custom', 'n'),
                            'year' => format_date(time(), 'custom', 'Y'));
  }

  $element['#tree'] = TRUE;

  // Determine the order of day, month, year in the site's chosen date format.
  $format = variable_get('date_format_short', 'm/d/Y - H:i');
  $sort = array();
  $sort['day'] = max(strpos($format, 'd'), strpos($format, 'j'));
  $sort['month'] = max(strpos($format, 'm'), strpos($format, 'M'));
  $sort['year'] = strpos($format, 'Y');
  asort($sort);
  $order = array_keys($sort);

  // Output multi-selector for date.
  foreach ($order as $type) {
    switch ($type) {
      case 'day':
        $options = drupal_map_assoc(range(1, 31));
        break;
      case 'month':
        $options = drupal_map_assoc(range(1, 12), 'map_month');
        break;
      case 'year':
        $options = drupal_map_assoc(range($first_year, $current_year));
        break;
    }
    $parents = $element['#parents'];
    $parents[] = $type;
    $element[$type] = array(
      '#type' => 'select',
      '#value' => $element['#value'][$type],
      '#attributes' => $element['#attributes'],
      '#options' => $options,
    );
  }

  return $element;
}